@@ -2,8 +2,7 @@ import time
 
 import pulsar
 from pulsar.apps.test import unittest, run_on_arbiter
-from pulsar.apps.shell import InteractiveConsole, decode_line
-from .pshell import PulsarShell
+from pulsar.apps.shell import InteractiveConsole, decode_line, PulsarShell
 
 
 class DummyConsole(InteractiveConsole):
@@ -15,16 +14,14 @@ class DummyConsole(InteractiveConsole):
         time.sleep(timeout)
         
 def start(actor):
-    return PulsarShell(console_class=DummyConsole).start()
+    return PulsarShell(console_class=DummyConsole, workers=2).start()
     
 
 class TestShell(unittest.TestCase):
     app = None
     @classmethod
     def setUpClass(cls):
-        outcome = pulsar.send('arbiter', 'run', start)
-        yield outcome
-        cls.app = outcome.result
+        cls.app = yield pulsar.send('arbiter', 'run', start)
         
     @classmethod
     def tearDownClass(cls):
@@ -33,17 +30,17 @@ class TestShell(unittest.TestCase):
             
     def testApp(self):
         app = self.app
-        self.assertEqual(app.name, 'pulsarshell')
+        self.assertEqual(app.name, 'shell')
         self.assertEqual(app.callable, None)
-        self.assertTrue(app.can_kill_arbiter)
-        self.assertEqual(app.console_class, DummyConsole)
-        self.assertEqual(app.cfg.concurrency, 'thread')
+        self.assertEqual(app.cfg.console_class, DummyConsole)
+        #self.assertEqual(app.cfg.workers, 1)
+        #self.assertEqual(app.cfg.concurrency, 'thread')
         self.assertEqual(decode_line('bla'), 'bla')
         
     @run_on_arbiter
     def testTestWorker(self):
         arbiter = pulsar.get_actor()
-        monitor = arbiter.monitors['pulsarshell']
+        monitor = arbiter.get_actor('shell')
         while not monitor.managed_actors:
             yield pulsar.NOT_DONE
         self.assertEqual(len(monitor.managed_actors), 1)
\ No newline at end of file
