@@ -43,10 +43,13 @@ The ``decode`` method must return a **two elements tuple** of the form:
 
 The ``encode`` method must return ``bytes`` which will be sent to the server.
 
-Useful settings
-==================
-All standard :ref:`settings <settings>` can be applied to a
-:class:`SocketServer`. These are some of the most important:
+.. _socket-server-settings:
+
+Socket Server Settings
+==============================
+All standard :ref:`application settings <settings>` can be applied to a
+:class:`SocketServer`. In addition, the following are
+specific to sockets and can be used to fine tune your application:
 
 bind
 ------
@@ -65,13 +68,13 @@ To control the concurrency of the server you can use the
     
 will serve a maximum of 1000 clients per :class:`pulsar.Worker` concurrently.
 
-keepalive
+keep_alive
 ---------------
 To control how long a :class:`pulsar.AsyncConnection` is kept alive after the
 last read from the remote client, one can use the
-:ref:`keepalive <setting-keepalive>` setting.
+:ref:`keepalive <setting-keep_alive>` setting.
 
-    python script.py --keepalive 10
+    python script.py --keep-alive 10
     
 will close a client connection which has been idle for 10 seconds.
  
@@ -96,7 +99,6 @@ In addition, a :class:`SocketServer` in multi-process mode is only available for
  * Windows running python 3.2 or above.
 '''
 import pulsar
-from pulsar import AsyncIOStream
 
 
 class Bind(pulsar.Setting):
@@ -106,21 +108,23 @@ class Bind(pulsar.Setting):
     flags = ["-b", "--bind"]
     meta = "ADDRESS"
     default = "127.0.0.1:{0}".format(pulsar.DEFAULT_PORT)
+    can_prefix = True
     desc = """\
         The socket to bind.
         
-        A string of the form: 'HOST', 'HOST:PORT', 'unix:PATH'. An IP is a valid
-        HOST.
+        A string of the form: ``HOST``, ``HOST:PORT``, ``unix:PATH``.
+        An IP is a valid HOST.
         """
         
-class Keepalive(pulsar.Setting):
+class KeepAlive(pulsar.Setting):
     section = "Socket Servers"
     app = 'socket'
-    name = "keepalive"
+    name = "keep_alive"
     flags = ["--keep-alive"]
     validator = pulsar.validate_pos_int
     type = int
     default = 15
+    can_prefix = True
     desc = """\
         The number of seconds to keep an idle client connection
         open."""
@@ -129,59 +133,44 @@ class Keepalive(pulsar.Setting):
 class SocketServer(pulsar.Application):
     '''This application bind a socket to a given address and listen for
 requests. The request handler is constructued from the
-:attr:`socket_server_factory` parameter/attribute.
-    
-.. attribute:: socket_server_factory
-
-    Callable which returns the asynchronous socket server, usually
-    a subclass of :class:`pulsar.AsyncSocketServer`.
+:attr:`handler` parameter/attribute.
     
 .. attribute:: address
 
     The socket address, available once the application has started.
     '''
-    _app_name = 'socket'
-    socket_server_factory = None
+    name = 'socket'
     address = None
+    cfg = pulsar.Config(apps=['socket'])
+    
+    def protocol_consumer(self):
+        '''Returns the callable application handler which is stored in
+:attr:`Worker.app_handler`, used by :class:`Worker` to carry out its task.
+By default it returns the :attr:`Application.callable`.'''
+        return self.callable
     
     def monitor_start(self, monitor):
         # if the platform does not support multiprocessing sockets set
         # the number of workers to 0.
         cfg = self.cfg
-        if not self.socket_server_factory:
-            raise TypeError('Socket server class not specified.')
         if not pulsar.platform.has_multiProcessSocket\
             or cfg.concurrency == 'thread':
             cfg.set('workers', 0)
         # Open the socket and bind to address
         address = self.cfg.address
         if address:
-            socket = pulsar.create_socket(address, backlog=self.cfg.backlog)
+            sock = pulsar.create_socket(address, bindto=True,
+                                        backlog=self.cfg.backlog)
         else:
             raise pulsar.ImproperlyConfigured('Could not open a socket. '
                                               'No address to bind to')
-        self.logger.info('Listening on %s', socket)
-        monitor.params.socket = socket
-        self.address = socket.name
+        self.logger.info('Listening on %s', sock)
+        monitor.params.sock = sock
+        self.address = sock.address
     
     def worker_start(self, worker):
         # Start the worker by starting the socket server
-        s = self.socket_server_factory(worker, worker.params.socket,
-                                       timeout=self.cfg.keepalive)
-        # We add the file descriptor handler
-        s.on_connection_callbacks.append(worker.handle_fd_event)
-        worker.socket_server = s
-    
-    def worker_stop(self, worker):
-        if hasattr(worker, 'socket_server'):
-            # we don't shut down the socket, simply remove all active
-            # connections and othe clean up operations.
-            worker.socket_server.on_close()
-        
-    def on_event(self, worker, fd, events):
-        connection = worker.socket_server.accept()
-        if connection is not None:
-            return connection.on_closed
+        worker.servers[self.name] = self.create_server(worker)
     
     def on_info(self, worker, data):
         server = worker.socket_server
@@ -190,3 +179,19 @@ requests. The request handler is constructued from the
                           'active_connections': server.active_connections,
                           'received_connections': server.received}
         return data
+
+    def create_server(self, worker):
+        '''Create the Server Protocol which will listen for requests. It
+uses the :meth:`handler` as its response protocol.'''
+        cfg = self.cfg
+        server = worker.event_loop.create_server(
+                                      sock=worker.params.sock,
+                                      consumer_factory=self.protocol_consumer(),
+                                      max_connections=cfg.max_requests,
+                                      timeout=cfg.keep_alive)
+        server.bind_event('connection_made', cfg.connection_made)
+        server.bind_event('pre_request', cfg.pre_request)
+        server.bind_event('post_request', cfg.post_request)
+        server.bind_event('connection_lost', cfg.connection_lost)
+        return server
+        
\ No newline at end of file
