@@ -85,6 +85,13 @@ Loading test cases is accomplished via the :class:`loader.TestLoader` class. In
 this context we refer to an ``object`` as a ``module`` (including a
 directory module) or a ``class``.
 
+.. note::
+
+    Test modules are specified when initialising a :class:`TestSuite`, by
+    passing the ``modules`` tuple/list of python dotted paths. If not defined
+    the default is to load test cases from the ``tests`` module.
+    
+
 These are the rules for loading tests:
 
 * Directories that aren't packages are not inspected. This means that if a directory
@@ -231,6 +238,7 @@ API
 
 .. module:: pulsar.apps.test
 
+
 Test Plugin
 ~~~~~~~~~~~~~~~~~~~
 
@@ -241,7 +249,6 @@ Test Plugin
 .. _unittest2: http://pypi.python.org/pypi/unittest2
 .. _mock: http://pypi.python.org/pypi/mock
 '''
-__test__ = False
 import logging
 import time
 
@@ -278,6 +285,8 @@ from .utils import *
 from .wsgi import *
 
 def dont_run_with_thread(obj):
+    '''Decorator for disabling test cases when the test suite runs in
+threading, rather than processing, mode.'''
     actor = pulsar.get_actor()
     if actor:
         d = unittest.skipUnless(actor.cfg.concurrency=='process',
@@ -378,10 +387,11 @@ is a group of tests specified in a test class.
                         plugins=(),
                         logconfig={
                                    'loggers': {
-                                LOGGER.name: {'handlers': ['console_message'],
-                                              'level': logging.INFO}
-                                               }
-                                   })
+                                   'pulsar.test': {
+                                        'handlers': ['console_message'],
+                                        'level': logging.WARNING}
+                                    }
+                        })
     @local_property
     def runner(self):
         '''Instance of :class:`TestRunner` driving the test case
@@ -395,7 +405,8 @@ configuration and plugins.'''
             raise ExitTest(str(abort_message))
         return runner
 
-    def on_config(self):
+    @local_property
+    def loader(self):
         #When config is available load the tests and check what type of
         #action is required.
         modules = self.cfg.get('modules')
@@ -405,8 +416,10 @@ configuration and plugins.'''
             modules = ['tests']
         if hasattr(modules, '__call__'):
             modules = modules(self)
-        loader = TestLoader(self.root_dir, modules, runner, logger=self.logger)
-        # Listing labels
+        return TestLoader(self.root_dir, modules, runner, logger=self.logger)
+    
+    def on_config(self):
+        loader = self.loader
         if self.cfg.list_labels:
             tags = self.cfg.labels
             if tags:
@@ -425,8 +438,7 @@ configuration and plugins.'''
                 print(tag)
             print('\n')
             return False
-        self.local.loader = loader
-
+    
     def monitor_start(self, monitor):
         '''When the monitor starts load all test classes into the queue'''
         super(TestSuite, self).monitor_start(monitor)
@@ -450,8 +462,8 @@ configuration and plugins.'''
             print(str(e))
             monitor.arbiter.stop()
         except Exception:
-            LOGGER.critical('Error occurred before starting tests',
-                            exc_info=True)
+            self.logger.critical('Error occurred before starting tests',
+                                 exc_info=True)
             monitor.arbiter.stop()
 
     @classmethod
