@@ -9,6 +9,13 @@ __all__ = ['WrapTest',
            'TestOption',
            'TestPlugin']
 
+
+def as_test_setting(setting):
+    setting.app = 'test'
+    setting.section = "Test"
+    return setting
+
+
 class TestOption(pulsar.Setting):
     virtual = True
     app = 'test'
@@ -43,11 +50,6 @@ class WrapTest(object):
         # This is the actual function to implement
         return self.testMethod()
     
-
-def as_test_setting(setting):
-    setting.app = 'test-plugin'
-    setting.section = "Test"
-    return setting
     
 class TestPluginMeta(type):
     
@@ -92,13 +94,51 @@ class TestPluginMeta(type):
         return super(TestPluginMeta, cls).__new__(cls, name, bases, attrs)
                 
     
-class TestPlugin(TestPluginMeta('TestPluginBase',(Plugin,),{'virtual': True})):
-    '''Base class for plugins which can be added to a :class:`TestSuite`
-to extend its functionalities. :class:`pulsar.Setting` are specified as
-class attributes and collected into the :attr:`config` attribute.
+class TestPlugin(TestPluginMeta('TestPluginBase', (Plugin,), {'virtual': True})):
+    '''Base class for :class:`result.Plugin` which can be added to a
+:class:`TestSuite` to extend its functionalities. :ref:`Settings <settings>`
+can be specified as class attributes and collected into the
+:attr:`config` attribute.
 
-If the :attr:`name` is not specified or its value validate as True,
-an additional settings is added to the configuration.
+If the class attribute :attr:`name` is not specified or its value validate as
+``True``, an additional :ref:`setting <settings>` is added to the configuration.
+In addition, a :class:`TestPlugin` can specify several additional
+:ref:`settings <settings>` as class attributes. For example, the
+:ref:`benchmark plugin <bench-plugin>` has an additional setting
+for controlling the number of repetitions::
+
+    class Bench(TestPlugin):
+        desc = "Run benchmarks function flagged with __benchmark__ attribute"
+        repeat = pulsar.Setting(type=int,
+                            default=1,
+                            validator=pulsar.validate_pos_int,
+                            desc="Default number of repetition")
+
+
+.. attribute:: name
+
+    Class attribute used for adding the default plugin :ref:`setting <settings>`
+    to the configuration container of the test suite application.
+    If the attribute is not set, the class name in lower case is used.
+    If set and validate as not ``True``, no new :ref:`setting <settings>`
+    is added to the test suite configuration parameters. For example::
+    
+        class MyPlugin(TestPlugin):
+            name = None
+            
+    won't add the default plugin :ref:`setting <settings>`.
+    
+.. attribute:: desc
+
+    Class attribute used as the description of the :ref:`setting <settings>`.
+    If :attr:`name` is disabled, this attribute is not relevant.
+    
+.. attribute:: config
+
+    A :class:`pulsar.utils.config.Config` container created by th
+    :class:`TestPlugin` metaclass. It collects the default setting, if
+    available, and any additional :ref:`settings <setting>` specified
+    as class attributes.
 '''
     virtual = True
     
@@ -109,7 +149,4 @@ an additional settings is added to the configuration.
         
     def configure(self, cfg):
         self.config = cfg
-        
-    def include_settings(self):
-        return [s.name for settinng in self.settings]
     
\ No newline at end of file
