@@ -1,8 +1,53 @@
 '''The :mod:`pulsar.apps.wsgi.content` introduces several utility classes
-handling asynchronous content on a WSGI server. The main class of this module
-is the :class:`AsyncString`. Web framework are smart ways of concatenating 
-bytes which need to be displayed on a web page or returned as a response to
-and HTTP client.
+handling asynchronous content on a WSGI server.
+
+A web framework is set of tools working together to concatenate
+``strings`` to return as a response to and HTTP client.
+A string can be Html, Json, plain text, XML or any other valid HTTP
+content type.
+
+The main class of this module is the :class:`AsyncString`, which can be
+considered as the atomic component of an asynchronous web framework.
+It is a smart way for concatenating asynchronous strings::
+
+    >>> string = AsyncString('Hello')
+    >>> string.render()
+    'Hello'
+    >>> string.render()
+    ...  
+    RuntimeError: AsyncString already streamed
+    
+An :class:`AsyncString` can only be rendered once, and it accepts
+:ref:`asynchronous components  <tutorials-coroutine>`::
+
+    >>> a = Deferred()
+    >>> string = AsyncString('Hello ', a)
+    >>> value = string.render()
+    >>> value
+    StreamRenderer
+    >>> value.done()
+    False
+    >>> 
+
+The :class:`StreamRenderer` is a specialised :class:`pulsar.Deferred` which
+results in a string.
+
+    >>> a.callback('World!')
+    'World!'
+    >>> value.done()
+    True
+    >>> value.result
+    'Hello World!'
+
+.. note::
+
+    The :meth:`AsyncString.do_stream` method is responsible for the streaming
+    of ``strings`` or :ref:`asynchronous components  <tutorials-coroutine>`.
+    It can be overwritten by subclasses to customise the way an
+    :class:`AsyncString` streams its :attr:`AsyncString.children`.
+    
+    The :meth:`AsyncString.to_string` method is responsible for the
+    concatenation of ``strings``. it can be customised by subclasses.
 
 Asynchronous String
 =====================
@@ -81,8 +126,7 @@ from copy import copy
 
 from pulsar import Deferred, multi_async, is_async, maybe_async, is_failure, async
 from pulsar.utils.pep import iteritems, is_string, ispy3k
-from pulsar.utils.structures import AttributeDictionary, recursive_update,\
-                                    OrderedDict
+from pulsar.utils.structures import AttributeDictionary, OrderedDict
 from pulsar.utils.html import slugify, INLINE_TAGS, tag_attributes, attr_iter,\
                                 csslink, dump_data_value, child_tag
 from pulsar.utils.httpurl import remove_double_slash, urljoin
@@ -144,11 +188,11 @@ pulsar WSGI servers.
     '''Content type for this :class:`AsyncString`'''
     encoding = None
     '''Charset encoding for this :class:`AsyncString`'''
+    _streamed = False
+    _children = None
+    _parent = None
     
     def __init__(self, *children):
-        self._streamed = False
-        self._children = []
-        self._parent = None
         for child in children:
             self.append(child)
     
@@ -164,22 +208,45 @@ pulsar WSGI servers.
 be other :class:`AsyncString` or string or bytes, depending on implementation.
 :attr:`children` are added and removed via the :meth:`append` and
 :meth:`remove` methods.'''
-        return copy(self._children)
+        if self._children is None:
+            self._children = []
+        return self._children
     
     def __repr__(self):
-        return self.__class__.__name__()
+        return self.__class__.__name__
     
     def __str__(self):
         return self.__repr__()
     
     def append(self, child):
-        '''Append ``child`` to the list of :attr:`children` of this
-:class:`AsyncString`.
+        '''Append ``child`` to the list of :attr:`children`.
 
 :param child: String, bytes or another :class:`AsyncString`. If it is an
     :class:`AsyncString`, this instance will be set as its :attr:`parent`.
     If ``child`` is ``None``, this method does nothing.
     
+'''
+        self.insert(None, child)
+        
+    def prepend(self, child):
+        '''Prepend ``child`` to the list of :attr:`children`.
+        
+This is a shortcut for the :meth:`insert` method at index 0.
+
+:param child: String, bytes or another :class:`AsyncString`. If it is an
+    :class:`AsyncString`, this instance will be set as its :attr:`parent`.
+    If ``child`` is ``None``, this method does nothing.
+    '''
+        self.insert(0, child)
+    
+    def insert(self, index, child):
+        '''Insert ``child`` into the list of :attr:`children` at ``index``.
+
+:param index: The index (positive integer) where to insert ``child``.
+:param child: String, bytes or another :class:`AsyncString`. If it is an
+    :class:`AsyncString`, this instance will be set as its :attr:`parent`.
+    If ``child`` is ``None``, this method does nothing.
+    
 '''
         # make sure that child is not in child
         if child is not None:
@@ -193,12 +260,15 @@ be other :class:`AsyncString` or string or bytes, depending on implementation.
                     # remove child from the child parent
                     child_parent.remove(child)
                 child._parent = self
-            self._children.append(child)
-        
+            if index is None:
+                self.children.append(child)
+            else:
+                self.children.insert(index, child)
+             
     def remove(self, child):
-        '''Remove a ``child`` from the list of :attr:``children``.'''
+        '''Remove a ``child`` from the list of :attr:`children`.'''
         try:
-            self._children.remove(child)
+            self.children.remove(child)
             if isinstance(child, AsyncString):
                 child._parent = None
         except ValueError:
@@ -206,10 +276,11 @@ be other :class:`AsyncString` or string or bytes, depending on implementation.
     
     def remove_all(self):
         '''Remove all :attr:`children`.'''
-        for child in self._children:
-            if isinstance(child, AsyncString):
-                child._parent = None
-        self._children = []
+        if self._children:
+            for child in self._children:
+                if isinstance(child, AsyncString):
+                    child._parent = None
+            self._children = []
         
     def append_to(self, parent):
         '''Append itself to ``parent``. Return ``self``.'''
@@ -218,6 +289,7 @@ be other :class:`AsyncString` or string or bytes, depending on implementation.
     
     def content(self, request=None):
         '''Return the :class:`StreamRenderer` for this instance.
+        
 This method can be called once only since it invokes the :meth:`stream`
 method.'''
         res = self.stream(request)
@@ -227,25 +299,39 @@ method.'''
             return StreamRenderer(res, self.to_string)
     
     def stream(self, request):
-        '''This is the most important method of an :class:`AsyncString`.
+        '''An iterable over strings or asynchronous elements.
+        
+This is the most important method of an :class:`AsyncString`.
 It is called by :meth:`content` or by the :attr:`parent` of this
-:class:`AsyncString`. It returns an iterable (list, tuple, generator) over
-strings, that means ``unicode/str`` for python 2, and ``str`` for python 3 or
-:ref:`asynchronous elements <tutorials-coroutine>` which result in strings.
-This method can be called once only, otherwise a
-:class:`RuntimeError` occurs.'''
+:class:`AsyncString`. It returns an iterable (list, tuple or a generator) over
+strings (``unicode/str`` for python 2, ``str`` only for python 3) or
+:ref:`asynchronous elements <tutorials-coroutine>` which result in
+strings. This method can be called **once only**, otherwise a
+:class:`RuntimeError` occurs.
+
+This method should not be overwritten, instead one should use the
+:meth:`do_stream` to customise behaviour.'''
         if self._streamed:
             raise RuntimeError('%s already streamed' % self)
-        if request:
-            self.stream_started(request)
         self._streamed = True
         return self.do_stream(request)
     
-    def stream_started(self, request):
-        '''Hook called if ``request`` is available, just before the
-:meth:`do_stream` is executed.'''
-        pass
-    
+    def do_stream(self, request):
+        '''Perform the actual streaming.
+        
+It must return an iterable over ``strings`` or
+:ref:`asynchronous elements <tutorials-coroutine>` which result in strings.
+
+This method can be re-implemented by subclasses and should not be invoked
+directly. Use the :meth:`stream` method instead.'''
+        if self._children:
+            for child in self._children:
+                if isinstance(child, AsyncString):
+                    for bit in child.stream(request):
+                        yield bit
+                else:
+                    yield child
+                
     @async()
     def http_response(self, request):
         '''Return a, possibly, :ref:`asynchronous WSGI iterable <wsgi-async>`.
@@ -256,16 +342,6 @@ starts the wsgi response.'''
         body = yield self.content(request)
         response.content = body
         yield response
-        
-    def do_stream(self, request):
-        '''Perform the actual streaming of this :class:`AsyncString`.
-This method can be re-implemented by subclasses.'''
-        for child in self._children:
-            if isinstance(child, AsyncString):
-                for bit in child.stream(request):
-                    yield bit
-            else:
-                yield child
                 
     def to_string(self, stream):
         '''Once the :class:`StreamRenderer`, returned by :meth:`content`
@@ -286,10 +362,7 @@ This is useful during testing. It is the synchronous equivalent of
         value = maybe_async(self.content(request))
         if is_failure(value):
             value.raise_all()
-        elif is_async(value):
-            raise ValueError('Could not render. Asynchronous value')
-        else:
-            return value
+        return value
             
 
 class Json(AsyncString):
@@ -297,8 +370,9 @@ class Json(AsyncString):
 The :attr:`AsyncString.content_type` attribute is set to
 ``application/json``.'''
     def __init__(self, *children, **params):
-        super(Json, self).__init__(*children)
         self.parameters = AttributeDictionary(params)
+        for child in children:
+            self.append(child)
         
     @property
     def json(self):
@@ -331,10 +405,11 @@ dictionary of ``defaults`` parameters. For example::
         p.update(params)
         return Html(tag, *children, **p)
     return html_input
-    
+        
     
 class Html(AsyncString):
-    '''An :class:`AsyncString` for html elements.
+    '''An :class:`AsyncString` for html strings.
+    
 The :attr:`AsyncString.content_type` attribute is set to ``text/html``.
 
 :param tag: Set the :attr:`tag` attribute. Must be given and can be ``None``.
@@ -355,32 +430,50 @@ Any other keyed-value parameter will be added as attribute, if in the set of
 '''
     def __init__(self, tag, *children, **params):
         self._tag = tag
-        self._classes = set()
-        self._data = {}
-        self._attr = {}
-        self._css = {}
+        self._extra = {}
         self._setup(**params)
-        super(Html, self).__init__(*children)
+        for child in children:
+            self.append(child)
         
     @property
     def content_type(self):
         return 'text/html'
-    
-    @property
-    def classes(self):
-        '''Set of classes for this :class:`Html` element.'''
-        return self._classes
-    
+        
     @property
     def tag(self):
         '''The tag for this HTML element, ``div``, ``a``, ``table`` and so
 forth. It can be ``None``.'''
         return self._tag
     
+    @property
+    def _classes(self):
+        if 'classes' in self._extra:
+            return self._extra['classes']
+        
+    @property
+    def _data(self):
+        if 'data' in self._extra:
+            return self._extra['data']
+        
+    @property
+    def _attr(self):
+        if 'attr' in self._extra:
+            return self._extra['attr']
+    
+    @property
+    def _css(self):
+        if 'css' in self._extra:
+            return self._extra['css']
+        
+    @property
+    def type(self):
+        if 'attr' in self._extra:
+            return self._extra['attr'].get('type')
+    
     @property
     def available_attributes(self):
         '''The list of valid HTML attributes for this :attr:`tag`.'''
-        return tag_attributes(self._tag, self._attr.get('type'))
+        return tag_attributes(self._tag, self.type)
     
     def get_form_value(self):
         '''Return the value of this :class:`Html` element when it is contained
@@ -431,15 +524,18 @@ in a Html form element. For most element it sets the ``value`` attribute.'''
     def attr(self, *args):
         '''Add the specific attribute to the attribute dictionary
 with key ``name`` and value ``value`` and return ``self``.'''
+        attr = self._attr
         if not args:
-            return self._attr
-        result, adding = self._attrdata(self._attr, *args)
+            return attr or {}
+        result, adding = self._attrdata('attr', *args)
         if adding:
+            if attr is None:
+                self._extra['attr'] = attr = {}
             available_attributes = self.available_attributes
             for name, value in iteritems(result):
                 if value is not None:
                     if name in available_attributes:
-                        self._attr[name] = value
+                        attr[name] = value
                     elif name is 'value':
                         self.append(value)
             result = self
@@ -447,76 +543,98 @@ with key ``name`` and value ``value`` and return ``self``.'''
     
     def data(self, *args):
         '''Add or retrieve data values for this :class:`Html`.'''
+        data = self._data
         if not args:
-            return self._data
-        result, adding = self._attrdata(self._data, *args)
+            return data or {}
+        result, adding = self._attrdata('data', *args)
         if adding:
-            recursive_update(self._data, result)
-            result = self
-        return result
+            if data is None:
+                self._extra['data'] = {}
+            add = self._visitor.add_data
+            for key, value in iteritems(result):
+                add(self, key, value)
+            return self
+        else:
+            return result
     
     def addClass(self, cn):
         '''Add the specific class names to the class set and return ``self``.'''
         if cn:
-            if isinstance(cn,(tuple,list,set,frozenset)):
+            if isinstance(cn, (tuple, list, set, frozenset)):
                 add = self.addClass
                 for c in cn:
                     add(c)
             else:
-                add = self._classes.add
+                classes = self._classes
+                if classes is None:
+                    self._extra['classes'] = classes = set()
+                add = classes.add
                 for cn in cn.split():
                     add(slugify(cn, rtx='-'))
         return self
 
     def hasClass(self, cn):
         '''``True`` if ``cn`` is a class of self.'''
-        return cn in self._classes
+        classes = self._classes
+        return classes and cn in classes
     
     def removeClass(self, cn):
         '''Remove classes'''
         if cn:
             ks = self._classes
-            for cn in cn.split():
-                if cn in ks:
-                    ks.remove(cn)
+            if ks:
+                for cn in cn.split():
+                    if cn in ks:
+                        ks.remove(cn)
         return self
     
     def flatatt(self, **attr):
         '''Return a string with atributes to add to the tag'''
         cs = ''
-        attr = self._attr.copy()
-        if self._classes:
-            cs = ' '.join(self._classes)
+        attr = self._attr
+        classes = self._classes
+        data = self._data
+        css = self._css
+        attr = attr.copy() if attr else {}
+        if classes:
+            cs = ' '.join(classes)
             attr['class'] = cs
-        if self._css:
+        if css:
             attr['style'] = ' '.join(('%s:%s;' % (k,v)\
-                                       for k,v in self._css.items()))
-        for k, v in self._data.items():
-            attr['data-%s' % k] = dump_data_value(v)
+                                       for k,v in css.items()))
+        if data:
+            for k, v in data.items():
+                attr['data-%s' % k] = dump_data_value(v)
         if attr:
             return ''.join(attr_iter(attr))
         else:
             return ''
 
     def css(self, mapping=None):
-        '''Update the css dictionary if *mapping* is a dictionary, otherwise
- return the css value at *mapping*.'''
+        '''Update the css dictionary if ``mapping`` is a dictionary, otherwise
+ return the css value at ``mapping``. If ``mapping`` is not given, return the
+ whole ``css`` dictionary if available.'''
+        css = self._css
         if mapping is None:
-            return self._css
+            return css
         elif isinstance(mapping, Mapping):
-            self._css.update(mapping)
+            if css is None:
+                self._extra['css'] = css = {}
+            css.update(mapping)
             return self
         else:
-            return self._css.get(mapping)
+            return css.get(mapping) if css else None
     
     def hide(self):
         '''Same as jQuery hide method.'''
-        self._css.update({'display': 'none'})
+        self.css({'display': 'none'})
         return self
     
     def show(self):
         '''Same as jQuery show method.'''
-        self._css.pop('display', None)
+        css = self._css
+        if css:
+            css.pop('display', None)
         return self
     
     def do_stream(self, request):
@@ -525,16 +643,19 @@ with key ``name`` and value ``value`` and return ``self``.'''
         else:
             if self._tag:
                 yield '<%s%s>' % (self._tag, self.flatatt())
-            for child in self._children:
-                if isinstance(child, AsyncString):
-                    for bit in child.stream(request):
-                        yield bit
-                else:
-                    yield child
+            if self._children:
+                for child in self._children:
+                    if isinstance(child, AsyncString):
+                        for bit in child.stream(request):
+                            yield bit
+                    else:
+                        yield child
             if self._tag:
                 yield '</%s>' % self._tag
     
     def _attrdata(self, cont, name, *val):
+        if not name:
+            return None, False
         if isinstance(name, Mapping):
             if val:
                 raise TypeError('Cannot set a value to %s' % name)
@@ -546,7 +667,8 @@ with key ``name`` and value ``value`` and return ``self``.'''
                 else:
                     raise TypeError('Too may arguments')
             else:
-                return cont.get(name), False
+                cont = self._extra.get(cont)
+                return cont.get(name) if cont else None, False
             
 
 class Media(AsyncString):
@@ -555,21 +677,25 @@ or scripts.
 
 .. attribute:: media_path
 
-    The path to the local media files, for example "/media/". Must
+    The base url path to the local media files, for example ``/media/``. Must
     include both slashes.
     
 .. attribute:: minified
 
-    Optional flag indicating if local media files should be modified to
-    end with ``.min.js`` or ``.min.css` rather than ``.js`` or ``.css``
+    Optional flag indicating if relative media files should be modified to
+    end with ``.min.js`` or ``.min.css`` rather than ``.js`` or ``.css``
     rispectively.
     
+    Default: ``False``
+    
 .. attribute:: known_libraries
 
     Optional dictionary of known media libraries, mapping a name to a
     valid absolute or local url. For example::
     
         known_libraries = {'jquery': '//code.jquery.com/jquery-1.9.1.min.js'}
+        
+    Default: ``None``
 '''
     mediatype = ('js', 'css')
     
@@ -578,14 +704,20 @@ or scripts.
         self.media_path = media_path
         self.minified = minified
         self.known_libraries = known_libraries or {}
-        self._children = OrderedDict()
+    
+    @property
+    def children(self):
+        if self._children is None:
+            self._children = OrderedDict()
+        return self._children
     
     def append(self, value):
         '''Append new media to the container.'''
         raise NotImplementedError
         
     def is_relative(self, path):
-        '''Check if ``path`` is a local relative path.'''
+        '''Check if ``path`` is a local relative path. A path is local relative
+when it does not start with a slash ``/`` nor ``http://`` nor ``https://``.'''
         if path.startswith('http://') or path.startswith('https://')\
                 or path.startswith('/'):
             return False
@@ -593,7 +725,15 @@ or scripts.
             return True
     
     def absolute_path(self, path):
-        '''Return a suitable url for ``path``.'''
+        '''Return a suitable absolute url for ``path``. The url is calculated
+in the following way:
+
+* Check if ``path`` is an entry in the :attr:`known_libraries` dictionary. In
+  this case replace ``path`` with ``known_libraries[path]``.
+* If ``path`` :meth:`is_relative` build a sutable url by prepending the 
+  :attr:`media_path` attribute.
+  
+:return: A url path to insert in a HTML ``link`` or ``script``.'''
         if path in self.known_libraries:
             path = self.known_libraries[path]
         if self.is_relative(path):
@@ -606,7 +746,7 @@ or scripts.
             return remove_double_slash('/%s/%s' % (self.media_path, path))
         else:
             return path
-
+        
     def _minify(self, path, postfix):
         new_postfix = '.min%s' % postfix
         if not path.endswith(new_postfix):
@@ -621,7 +761,7 @@ class Css(Media):
             if isinstance(value, str):
                 value = {'all': [value]}
             for media, values in value.items():
-                m = self._children.get(media, [])
+                m = self.children.get(media, [])
                 for value in values:
                     if not isinstance(value, (tuple, list)):
                         value = (value, None)
@@ -629,11 +769,12 @@ class Css(Media):
                     value = csslink(self.absolute_path(path), condition)
                     if value not in m:
                         m.append(value)
-                self._children[media] = m
+                self.children[media] = m
 
     def do_stream(self, request):
-        for medium in sorted(self._children):
-            paths = self._children[medium]
+        children = self.children
+        for medium in sorted(children):
+            paths = children[medium]
             medium = '' if medium == 'all' else " media='%s'" % medium
             for path in paths:
                 link = "<link href='%s' type='text/css'%s rel='stylesheet'/>\n"\
@@ -646,14 +787,17 @@ class Css(Media):
 class Scripts(Media):
         
     def append(self, child):
-        if child and is_string(child):
-            path = self.absolute_path(child)
-            script = Html('script', src=path, type='text/javascript')
-            self._children[script] = script
-            return script
+        if child:
+            if is_string(child):
+                path = self.absolute_path(child)
+                script = Html('script', src=path, type='application/javascript')
+                self.children[script] = script
+                return script
+            elif isinstance(child, Html) and child.tag == 'script':
+                self.children[child] = child
     
     def do_stream(self, request):
-        for child in self._children.values():
+        for child in self.children.values():
             for bit in child.stream(request):
                 yield bit
             yield '\n'
