@@ -96,6 +96,7 @@ files such ass ``css``, ``javascript``, images and so forth.
    
 .. _WSGI: http://www.wsgi.org
 '''
+import sys
 import os
 import re
 import stat
@@ -107,15 +108,16 @@ from pulsar.utils.httpurl import http_date, CacheControl
 from pulsar.utils.structures import AttributeDictionary, OrderedDict
 from pulsar.utils.log import LocalMixin, local_property
 from pulsar import Http404, PermissionDenied, HttpException, HttpRedirect,\
-                    async, is_async, multi_async
+                    async, is_async, multi_async, maybe_async
 
 from .route import Route
-from .utils import wsgi_request
+from .utils import wsgi_request, handle_wsgi_error
 from .content import Html
 from .wrappers import WsgiResponse
 
 __all__ = ['WsgiHandler', 'LazyWsgi', 'Router',
-           'MediaRouter', 'FileRouter', 'MediaMixin']
+           'MediaRouter', 'FileRouter', 'MediaMixin',
+           'RouterParam']
 
 
 class WsgiHandler(object):
@@ -147,25 +149,26 @@ class WsgiHandler(object):
 
     def __call__(self, environ, start_response):
         '''The WSGI callable'''
-        response = None
-        for middleware in self.middleware:
-            response = middleware(environ, start_response)
-            if response is not None:
-                break
-        if response is None:
-            raise Http404(environ.get('PATH_INFO','/'))
-        if is_async(response):
-            return response.add_callback(
-                        partial(self.finish, environ, start_response))
-        else:
-            return self.finish(environ, start_response, response)
+        return maybe_async(self._call(environ, start_response))
     
-    def finish(self, environ, start_response, response):
-        if isinstance(response, WsgiResponse):
+    def _call(self, environ, start_response):
+        resp = None
+        for middleware in self.middleware:
+            try:
+                resp = middleware(environ, start_response)
+            except Exception:
+                resp = handle_wsgi_error(environ, maybe_async(sys.exc_info()))
+            if resp is not None:
+                break
+        if resp is None:
+            raise Http404
+        if is_async(resp):
+            resp = yield resp.add_errback(partial(handle_wsgi_error, environ))
+        if isinstance(resp, WsgiResponse):
             for middleware in self.response_middleware:
-                response = middleware(environ, response)
-            start_response(response.status, response.get_headers())
-        return response
+                resp = yield middleware(environ, resp)
+            start_response(resp.status, resp.get_headers())
+        yield resp
     
 
 class LazyWsgi(LocalMixin):
@@ -203,13 +206,27 @@ def get_roule_methods(attrs):
             rule_methods.append((code, rule_method))
     return sorted(rule_methods, key=lambda x: x[1][3])
     
+
+class RouterParam(object):
     
+    def __init__(self, value):
+        self.value = value
+        
+        
 class RouterType(type):
     
     def __new__(cls, name, bases, attrs):
         rule_methods = get_roule_methods(attrs.items())
+        parameters = {}
+        for name, value in list(attrs.items()):
+            if isinstance(value, RouterParam):
+                parameters[name] = attrs.pop(name).value
         no_rule = set(attrs) - set((x[0] for x in rule_methods))
-        for base in bases[::-1]:
+        for base in reversed(bases):
+            if hasattr(base, 'parameters'):
+                params = base.parameters.copy()
+                params.update(parameters)
+                parameters = params
             if hasattr(base, 'rule_methods'):
                 items = base.rule_methods.items()
             else:
@@ -219,6 +236,7 @@ class RouterType(type):
             rule_methods = base_rules + rule_methods
         #                
         attrs['rule_methods'] = OrderedDict(rule_methods)
+        attrs['parameters'] = parameters
         return super(RouterType, cls).__new__(cls, name, bases, attrs)
     
 
@@ -255,32 +273,31 @@ request, the ``get(self, request)`` method must be implemented.
 .. attribute:: parent
 
     The parent :class:`Router` of this :class:`Router`.
-        
-.. attribute:: accept_content_types
-
-    Tuple of content type accepted by this :class:`Router`. It can be specified
-    as class attribute or during initialisation. This is an important parameter.
-    It is used to set the content type of the response instance.
-    Default ``None``.
     
 .. attribute:: parameters
 
-    A :class:`pulsar.utils.structures.AttributeDictionary` of parameters.
+    A :class:`pulsar.utils.structures.AttributeDictionary` of parameters for
+    this :class:`Router`. Parameters are created at initialisation from
+    the ``parameters`` class attribute and the key-valued parameters
+    passed to the ``__init__`` method for which the value is not callable.
 '''
-    creation_count = 0
-    accept_content_types = None
+    _creation_count = 0
     _parent = None
+    _name = None
+    
+    accept_content_types = RouterParam(None)
     
     def __init__(self, rule, *routes, **parameters):
-        self.__class__.creation_count += 1
-        self.creation_count = self.__class__.creation_count
+        Router._creation_count += 1
+        self._creation_count = Router._creation_count
         if not isinstance(rule, Route):
             rule = Route(rule)
         self.route = rule
+        self._name = parameters.pop('name', rule.rule)
         self.routes = []
         for router in routes:
             self.add_child(router)
-        self.parameters = AttributeDictionary()
+        self.parameters = AttributeDictionary(self.parameters)
         for name, rule_method in self.rule_methods.items():
             rule, method, params, count = rule_method
             rparameters = params.copy()
@@ -290,17 +307,18 @@ request, the ``get(self, request)`` method must be implemented.
                 handler.rule_method = rule_method
             router = self.add_child(Router(rule, **rparameters))
             setattr(router, method, handler)
-        if 'accept_content_types' in parameters:
-            self.accept_content_types = parameters.pop('accept_content_types')
-        self.setup(**parameters)
-        
-    def setup(self, content_type=None, **parameters):
         for name, value in parameters.items():
-            if not hasattr(self, name) and hasattr(value, '__call__'):
+            if hasattr(value, '__call__'):
                 setattr(self, name, value)
             else:
                 self.parameters[name] = value
     
+    @property
+    def name(self):
+        '''The name of this :class:`Router`. This attribute can be specified
+during initialisation.'''
+        return self._name
+    
     @property
     def root(self):
         '''The root :class:`Router` for this :class:`Router`.'''
@@ -313,19 +331,47 @@ request, the ``get(self, request)`` method must be implemented.
     def parent(self):
         return self._parent
     
-    def parent_property(self, name):
-        value = getattr(self, name, None)
-        if value is None and self._parent:
-            return self._parent.parent_property(name)
+    @property
+    def creation_count(self):
+        '''Integer for sorting :class:`Router` by creation.'''
+        return self._creation_count
+        
+    def path(self, **urlargs):
+        route = self.route
+        if self._parent:
+            route = self._parent.route + route
+        return route.url(**urlargs)
+    
+    def __getattr__(self, name):
+        '''Check the value of a :attr:`parameters` ``name``. If the parameter is
+        not available, retrieve the parameter from the :attr:`parent`
+        :class:`Router` if it exists.'''
+        if not name.startswith('_'):
+            return self.get_parameter(name, False)
+        self.no_param(name)
+        
+    def get_parameter(self, name, safe=True):
+        value = self.parameters.get(name)
+        if value is None:
+            if self._parent:
+                return self._parent.get_parameter(name, safe)
+            elif name in self.parameters:
+                return value
+            elif not safe:
+                self.no_param(name)
         else:
             return value
         
+    def no_param(self, name):
+        raise AttributeError("'%s' object has no attribute '%s'" %
+                             (self.__class__.__name__, name))
+        
     def content_type(self, request):
         '''Evaluate the content type for the response to the client ``request``.
 The method uses the :attr:`accept_content_types` tuple of accepted content
 types and the content types accepted by the client and figure out
 the best match.'''
-        accept = self.parent_property('accept_content_types')
+        accept = self.accept_content_types
         if accept:
             return request.content_types.best_match(accept)
        
@@ -339,6 +385,19 @@ the best match.'''
         if router_args:
             router, args = router_args
             return router.response(environ, start_response, args)
+        else:
+            if self.route.is_leaf:
+                if path.endswith('/'):
+                    router_args = self.resolve(path[:-1])
+                    if router_args is not None:
+                        return self.redirect(environ, start_response,
+                                             '/%s' % path[:-1])
+            else:
+                if not path.endswith('/'):
+                    router_args = self.resolve('%s/' % path)
+                    if router_args is not None:
+                        return self.redirect(environ, start_response,
+                                             '/%s/' % path)
         
     def resolve(self, path, urlargs=None):
         '''Resolve a path and return a ``(handler, urlargs)`` tuple or
@@ -346,16 +405,6 @@ the best match.'''
         urlargs = urlargs if urlargs is not None else {}
         match = self.route.match(path)
         if match is None:
-            if self.route.is_leaf:
-                if path.endswith('/'):
-                    match = self.route.match(path[:-1])
-                    if match is not None and not match:
-                        return Redirect(path[:-1]), None
-            else:
-                if not path.endswith('/'):
-                    match = self.route.match('%s/' % path)
-                    if match is not None:
-                        return Redirect('%s/' % path), None
             return
         if '__remaining__' in match:
             remaining_path = match['__remaining__']
@@ -386,6 +435,12 @@ to actually produce the WSGI response.'''
         environ['pulsar.cache'] = yield multi_async(request.cache)
         yield callable(request)
     
+    @async()
+    def redirect(self, environ, start_response, path):
+        request = wsgi_request(environ, self)
+        environ['pulsar.cache'] = yield multi_async(request.cache)
+        raise HttpRedirect(path)
+        
     def add_child(self, router):
         '''Add a new :class:`Router` to the :attr:`routes` list. If this
 :class:`Router` is a leaf route, add a slash to the url.'''
@@ -409,6 +464,12 @@ to actually produce the WSGI response.'''
              self.routes.remove(router)
              router._parent = None
         
+    def get_route(self, name):
+        '''Get a child :class:`Router` by its :attr:`name`.'''
+        for route in self.routes:
+            if route.name == name:
+                return route
+            
     def link(self, *args, **urlargs):
         '''Return an anchor :class:`Html` element with the `href` attribute
 set to the url of this :class:`Router`.'''
@@ -451,7 +512,8 @@ returns ``utf-8``.'''
 class MediaMixin(Router):
     accept_content_types = ('application/octet-stream', 'text/css')
     cache_control = CacheControl(maxage=86400)
-    
+    _file_path = ''
+        
     def serve_file(self, request, fullpath):
         # Respect the If-Modified-Since header.
         statobj = os.stat(fullpath)
@@ -539,16 +601,20 @@ directory.
         self._show_indexes = show_indexes
         self._file_path = path
         
+    def filesystem_path(self, request):
+        bits = request.urlargs['path'].split('/')
+        '''Retrieve the filesystem path for this request.'''
+        if len(bits) == 1 and bits[0] == '':
+            bits.pop(0)
+        return os.path.join(self._file_path, *bits)
+    
     def get(self, request):
-        paths = request.urlargs['path'].split('/')
-        if len(paths) == 1 and paths[0] == '':
-            paths.pop(0)
-        fullpath = os.path.join(self._file_path, *paths)
+        fullpath = self.filesystem_path(request)
         if os.path.isdir(fullpath):
             if self._show_indexes:
                 return self.directory_index(request, fullpath)
             else:
-                raise PermissionDenied()
+                raise PermissionDenied
         elif os.path.exists(fullpath):
             return self.serve_file(request, fullpath)
         else:
@@ -556,11 +622,18 @@ directory.
 
 
 class FileRouter(MediaMixin):
-    
+    '''A Router for a single file.'''
     def __init__(self, route, file_path):
         super(FileRouter, self).__init__(route)
         self._file_path = file_path
         
+    def filesystem_path(self, request):
+        return self._file_path
+        
     def get(self, request):
-        return self.serve_file(request, self._file_path)
+        fullpath = self.filesystem_path(request)
+        if os.path.isfile(fullpath):
+            return self.serve_file(request, fullpath)
+        else:
+            raise Http404
     
