@@ -163,13 +163,17 @@ attribute by exposing the :attr:`Setting.name` as attribute.
             self.import_from_module(config)
 
     def __getattr__(self, name):
-        return self.get(name)
+        try:
+            return self._get(name)
+        except KeyError:
+            raise AttributeError("'%s' object has no attribute '%s'." %
+                                 (self.__class__.__name__, name))
 
     def __setattr__(self, name, value):
         if name != "settings" and name in self.settings:
             raise AttributeError("Invalid access!")
         super(Config, self).__setattr__(name, value)
-
+    
     def update(self, data):
         '''Update this :attr:`Config` with ``data`` which is either an
 instance of Mapping or :class:`Config`.'''
@@ -181,18 +185,14 @@ instance of Mapping or :class:`Config`.'''
         '''Get the value at ``name`` for this :class:`Config` container
 following this algorithm:
 
-* check ``name`` in the :attr:`settings` dictionary.
-* if ``name`` is a known setting parameter return ``default``
-* check ``name`` in the :attr:`params` dictionary
-* raise :class:`AttributeError`.
+* returns the ``name`` value in the :attr:`settings` dictionary if available.
+* returns the ``name`` value in the :attr:`params` dictionary if available.
+* returns the ``default`` value.
 '''
-        if name not in self.settings:
-            if name in KNOWN_SETTINGS:
-                return default
-            if name in self.params:
-                return self.params[name]
-            raise AttributeError("No configuration setting for: %s" % name)
-        return self.settings[name].get()
+        try:
+            return self._get(name, default)
+        except KeyError:
+            return default
 
     def set(self, name, value, default=False):
         '''Set the configuration :class:`Setting` at *name* with a new
@@ -334,6 +334,15 @@ is given, it prefixes all non
                 if setting.app and setting.app not in self.apps:
                     continue    # the setting is for an app not in the apps set
             self.settings[setting.name] = setting
+            
+    def _get(self, name, default=None):
+        if name not in self.settings:
+            if name in KNOWN_SETTINGS:
+                return default
+            if name in self.params:
+                return self.params[name]
+            raise KeyError("'%s'" % name)
+        return self.settings[name].get()
     
             
 
@@ -467,19 +476,21 @@ as base class for other settings.'''
 
     def __init__(self, name=None, flags=None, action=None, type=None,
                  default=None, nargs=None, desc=None, validator=None,
-                 app=None):
+                 app=None, meta=None, choices=None):
+        self.app = app or self.app
+        self.choices = choices or self.choices
         self.default = default if default is not None else self.default
-        if self.default is not None:
-            self.set(self.default)
-        self.name = name or self.name
+        self.desc = desc or self.desc
         self.flags = flags or self.flags
         self.action = action or self.action
+        self.meta = meta or self.meta
+        self.name = name or self.name
         self.nargs = nargs or self.nargs
         self.type = type or self.type
-        self.desc = desc or self.desc
-        self.app = app or self.app
         self.short = self.short or self.desc
         self.desc = self.desc or self.short
+        if self.default is not None:
+            self.set(self.default)
         if self.app and not self.section:
             self.section = self.app
         if not self.section:
@@ -491,18 +502,21 @@ as base class for other settings.'''
     def __getstate__(self):
         return self.__dict__.copy()
 
-    def add_argument(self, parser):
+    def add_argument(self, parser, set_default=False):
         '''Add this :class:`Setting` to the *parser*, an instance of
 python :class:`argparse.ArgumentParser`, only if :attr:`flags` or
 :attr:`nargs` and :attr:`name` are defined.'''
+        default = self.default if set_default else None
         kwargs = {'nargs':self.nargs}
         if self.type and self.type != 'string':
             kwargs["type"] = self.type
+        if self.choices:
+            kwargs["choices"] = self.choices
         if self.flags:
             args = tuple(self.flags)
             kwargs.update({"dest": self.name,
                            "action": self.action or "store",
-                           "default": None,
+                           "default": default,
                            "help": "%s [%s]" % (self.short, self.default)})
             if kwargs["action"] != "store":
                 kwargs.pop("type",None)
